// Terrain Build Rules x32
// A script that enforces brick placement rules for a community terrain build

// This bit is here to make re-executing the script easier
if(isPackage(TerrainBuildRulesx32Package))
   deactivatePackage(TerrainBuildRulesx32Package);

package TerrainBuildRulesx32Package
{
   function ServerCmdPlantBrick(%client)
   {
      // Call the default version of the plant function
      // Compare the player's brick count before and after to see if plant succeeded
      %preCount = %client.brickGroup.getCount();
      Parent::ServerCmdPlantBrick(%client);
      %count = %client.brickGroup.getCount();
      if(%count <= %preCount)
         return;
      
      // The last brick in their brick group should be the one they just planted
      %plantedBrick = %client.brickGroup.getObject(%count - 1);
      
      // Special rules for the first brick planted, so we can get the build started
      if($Server::BrickCount <= 1)
      {
         // if they managed to plant the first brick not on the ground then something is wrong
         if(%plantedBrick.getDistanceFromGround() != 0)
         {
            error("ERROR: [TerrainBuildRulesx32Package]ServerCmdPlantBrick() - first brick somehow not planted on the ground");
            return;
         }
         
         // must be a 32x plate
         if(%plantedBrick.getDataBlock().getID() != brick32x32fData.getID())
         {
            CommandToClient(%client, 'MessageBoxOK', "Invalid Brick", "You must start with a 32x32 Base<bitmap:base/client/ui/brickIcons/32x32 Base>");
            %plantedBrick.killBrick();
            return;
         }
         
         return;
      }

      // They planted a brick.  Was it on the ground?
      if(%plantedBrick.getDistanceFromGround() != 0)
         return;

      // It was on the ground.  It better be a 32x baseplate.
      if(%plantedBrick.getDataBlock().getID() != brick32x32fData.getID())          //Note: comparing the two datablock ID numbers is faster than doing a string compare on their names
      {
         CommandToClient(%client, 'MessageBoxOK', "Invalid Brick", "You must start with a 32x32 Base<bitmap:base/client/ui/brickIcons/32x32 Base>");
         %plantedBrick.killBrick();
         return;
      }

      // Also, it better be lined up with an existing baseplate
      // Search for nearby plates and check their relative position
      %pos = %plantedBrick.getPosition();
      %box = "16.1 16.1 0.1";
      %mask = $TypeMasks::FxBrickAlwaysObjectType;

      %plantX = getWord(%plantedBrick.getPosition(), 0);
      %plantY = getWord(%plantedBrick.getPosition(), 1);

      initContainerBoxSearch(%pos, %box, %mask);
	   while (%searchObj = containerSearchNext())
	   {	
         if(%searchObj.getDataBlock().getID() != brick32x32fData.getID())
            continue;

         if(%searchObj.getDistanceFromGround() != 0)
            continue;

         if(!%searchObj.isPlanted())
            continue;
         
         if(%searchObj.isDead())
            continue;

         if(%searchObj == %plantedBrick)
            continue;

         //To be lined up, the x or y position must be the same
         %searchX = getWord(%searchObj.getPosition(), 0);
         %searchY = getWord(%searchObj.getPosition(), 1);

         if(mAbs(%searchX - %plantX) < 0.05 ||               // Note: We compare with subtraction like this because direct comparison with "==" on floating point numbers is not reliable
            mAbs(%searchY - %plantY) < 0.05 )
            return;
      }

      //If we got this far it must not have been lined up with any existing plates
      CommandToClient(%client, 'MessageBoxOK', "Invalid Placement", "You must line up your baseplate with the existing grid.");
      %plantedBrick.killBrick();
   }
};

activatePackage(TerrainBuildRulesx32Package);